@@ -7,7 +7,7 @@ setAs("ANY",    "denseMatrix", function(from) Matrix(from, sparse=FALSE))
 ## setAs("matrix", "denseMatrix", ....) which was slightly more efficient than
 ##  Matrix(.)  but would have many things in common
 
-setAs(from = "denseMatrix", to = "generalMatrix", as_geSimpl)
+setAs("denseMatrix", "generalMatrix", as_geSimpl)
 
 ## dense to sparse:
 ## : if we do this, do it "right", i.e. preserve symmetric/triangular!
@@ -16,19 +16,26 @@ setAs(from = "denseMatrix", to = "generalMatrix", as_geSimpl)
 ## ##      function(from) as(as(from, "dgeMatrix"), "dsparseMatrix"))
 ##       function(from) as(as(from, "dgeMatrix"), "dgCMatrix"))
 
-.dense2C <- function(from) {
-    cl <- class(from)
-    cld <- getClassDef(cl) ## get it once (speedup)
+.dense2C <- function(from, kind = NA, uplo = "U") {
+    useK <- is.character(kind) && length(kind) == 1 &&
+        kind %in% c("gen", "sym", "tri")
+    if(!useK) {
+        cl <- class(from)
+        cld <- getClassDef(cl) ## get it once (speedup)
+    }
     r <- .Call(dense_to_Csparse, from)# goes via "generalMatrix"
     ## FIXME: for symmetric / triangular matrices, this is a waste, notably if packed
-
-    if (extends(cld, "generalMatrix"))
+    if (useK && kind == "gen"  ||  !useK && extends(cld, "generalMatrix"))
 	r
-    else if(extends(cld, "symmetricMatrix"))
-	forceSymmetric(r)
-    else if(extends(cld, "diagonalMatrix"))
+    else if(useK && kind == "sym" || !useK && extends(cld, "symmetricMatrix"))
+	forceCspSymmetric(r, uplo, isTri = FALSE)
+    else if(!useK && extends(cld, "diagonalMatrix"))
 	stop("diagonalMatrix in .dense2C() -- should never happen, please report!")
     else { ## we have "triangular" :
+        if(useK) {
+            cl <- class(from)
+            cld <- getClassDef(cl) ## get it once (speedup)
+        }
 	if	(extends(cld,"dMatrix")) as(r, "dtCMatrix")
         else if (extends(cld,"lMatrix")) as(r, "ltCMatrix")
         else if (extends(cld,"nMatrix")) as(r, "ntCMatrix")
@@ -37,7 +44,7 @@ setAs(from = "denseMatrix", to = "generalMatrix", as_geSimpl)
     }
 }
 
-setAs("denseMatrix", "CsparseMatrix", .dense2C)
+setAs("denseMatrix", "CsparseMatrix", function(from) .dense2C(from))
 
 ## This sometimes fails (eg. for "lsyMatrix"), and we really want to
 ## use the generic ``go via Csparse'' (top of ./sparseMatrix.R) instead
@@ -191,25 +198,36 @@ setReplaceMethod("[", signature(x = "denseMatrix", i = "matrix",  # 2-col.matrix
 
 
 setMethod("isSymmetric", signature(object = "denseMatrix"),
-	  function(object, tol = 100*.Machine$double.eps, ...) {
+	  function(object, tol = 100*.Machine$double.eps, tol1 = 8*tol, ...) {
 	      ## pretest: is it square?
 	      d <- dim(object)
-	      if(d[1] != d[2]) return(FALSE)
+              if((n <- d[1L]) != d[2L]) return(FALSE)
+	      if(n <= 1L) return(TRUE)
+	      ## else: square (n x n) matrix, n >= 2 :
+              is.z <- is(object, "zMatrix")
+	      ## initial tests, fast for large non-symmetric:
+	      if(length(tol1)) {
+		  ## initial pre-tests, fast for large non-symmetric:
+		  Cj <- if(is.z) Conj else identity
+		  for(i in unique(c(1L, 2L, n-1L, n)))
+		      if(is.character(all.equal(object[i, ], Cj(object[, i]),
+						tolerance = tol1, ...))) return(FALSE)
+	      }
 	      ## else slower test
 	      if (is(object,"dMatrix"))
-		  isTRUE(all.equal(as(object, "dgeMatrix"),
-				   as(t(object), "dgeMatrix"),
-				   tolerance = tol, ...))
+		  isTRUE(all.equal(as(  object,  "dgeMatrix"),
+				   as(t(object), "dgeMatrix"), tolerance = tol, ...))
 	      else if (is(object, "nMatrix"))
-		  identical(as(object, "ngeMatrix"),
+		  identical(as(  object,  "ngeMatrix"),
 			    as(t(object), "ngeMatrix"))
 	      else if (is(object, "lMatrix"))# not possible currently
 		  ## test for exact equality; FIXME(?): identical() too strict?
-		  identical(as(object, "lgeMatrix"),
+		  identical(as(  object,  "lgeMatrix"),
 			    as(t(object), "lgeMatrix"))
-	      else if (is(object, "zMatrix")) ## will error out here
-		  identical(as(object, "zgeMatrix"),
-			    as(t(object), "zgeMatrix"))
+	      else if (is.z) ## will error out here
+		  isTRUE(all.equal(as(       object,   "zgeMatrix"),
+				   as(Conj(t(object)), "zgeMatrix"),
+				   tolerance = tol, ...))
 	      else if (is(object, "iMatrix")) ## will error out here
 		  identical(as(object, "igeMatrix"),
 			    as(t(object), "igeMatrix"))
