@@ -120,6 +120,7 @@ if(FALSE)##--- no longer used:
     ## block-diagonal matrix [a dgTMatrix] from list of matrices
     stopifnot(is.list(lst), (nl <- length(lst)) >= 1)
 
+### FIXME: next line is *slow* when lst = list of 75'000  dense 3x3 matrices
     Tlst <- lapply(lapply(lst, as_Csp2), # includes "diagU2N"
 		   as, "TsparseMatrix")
     if(nl == 1) return(Tlst[[1]])
@@ -657,7 +658,10 @@ diagmatprod <- function(x, y) {
     ## x is diagonalMatrix
     dy <- dim(y)
     if(x@Dim[2L] != dy[1L]) stop("non-matching dimensions")
-    Matrix(if(x@diag == "U") y else x@x * y, nrow=dy[1L], ncol=dy[2L])
+    if(prod(dy))
+	Matrix(if(x@diag == "U") y else x@x * y)
+    else
+	Matrix(if(x@diag == "U") y else x@x * y, nrow=dy[1L], ncol=dy[2L])
 }
 setMethod("%*%", signature(x = "diagonalMatrix", y = "matrix"), diagmatprod)
 
@@ -666,7 +670,10 @@ diagmatprod2 <- function(x, y=NULL, boolArith = NA, ...) {
     ## x is diagonalMatrix
     dy <- dim(y)
     if(x@Dim[2L] != dy[1L]) stop("non-matching dimensions")
-    Matrix(if(x@diag == "U") y else x@x * y, nrow=dy[1L], ncol=dy[2L])
+    if(prod(dy))
+	Matrix(if(x@diag == "U") y else x@x * y)
+    else
+	Matrix(if(x@diag == "U") y else x@x * y, nrow=dy[1L], ncol=dy[2L])
 }
 setMethod("crossprod",  signature(x = "diagonalMatrix", y = "matrix"), diagmatprod2)
 
