@@ -4,6 +4,7 @@ library(Matrix)
 
 source(system.file("test-tools.R", package = "Matrix"))# identical3() etc
 doExtras
+(is64bit <- .Machine$sizeof.pointer == 8)
 ### "sparseQR" : Check consistency of methods
 ##   --------
 data(KNex); mm <- KNex$mm; y <- KNex$y
@@ -11,35 +12,41 @@ stopifnot(is((Y <- Matrix(y)), "dgeMatrix"))
 md <- as(mm, "matrix")                  # dense
 
 (cS <- system.time(Sq <- qr(mm)))
-(cD <- system.time(Dq <- qr(md))) # 1.1 sec. (lynne, 2011)
+(cD <- system.time(Dq <- qr(md))) # 1.1 sec. (lynne, 2011); 0.535 sec. (lynne, 2014)
 cD[1] / cS[1] # dense is  much ( ~ 100--170 times) slower
 
+## MM__FIXME__: move these functions to ../inst/test-tools.R
 chkQR <- function(a,
                   y = seq_len(nrow(a)),## RHS: made to contain no 0
-                  a.qr = qr(a), tol = 1e-13, # 1e-13 failing very rarely (interesting)
+                  a.qr = qr(a), tol = 1e-11, # 1e-13 failing very rarely (interesting)
                   ##----------
                   Qinv.chk = !sp.rank.def, QtQ.chk = !sp.rank.def,
-                  verbose=getOption("verbose"))
+                  verbose = getOption("Matrix.verbose", FALSE), giveRE = verbose,
+                  quiet = FALSE)
 {
     d <- dim(a)
     stopifnot((n <- d[1]) >= (p <- d[2]), is.numeric(y))
     kind <- if(is.qr(a.qr)) "qr" else
     if(is(a.qr, "sparseQR")) "spQR" else stop("unknown qr() class: ", class(a.qr))
-
+    if(!missing(verbose) && verbose) {
+	op <- options(Matrix.verbose = verbose)
+	on.exit(options(op))
+    }
     ## rank.def <- switch(kind,
     ##     	       "qr"  = a.qr$rank < length(a.qr$pivot),
     ##     	       "spQR" = a.qr@V@Dim[1] > a.qr@Dim[1])
     sp.rank.def <- (kind == "spQR") && (a.qr@V@Dim[1] > a.qr@Dim[1])
-    if(sp.rank.def && (missing(Qinv.chk) || missing(QtQ.chk)))
+    if(sp.rank.def && !quiet && (missing(Qinv.chk) || missing(QtQ.chk)))
 	message("is sparse *structurally* rank deficient:  Qinv.chk=",
 		Qinv.chk,", QtQ.chk=",QtQ.chk)
     if(is.na(QtQ.chk )) QtQ.chk  <- !sp.rank.def
     if(is.na(Qinv.chk)) Qinv.chk <- !sp.rank.def
 
-    if(Qinv.chk) ## qr.qy and qr.qty should be inverses,	 Q'Q y = y = QQ' y :
-        ## FIXME: Fails for structurally rank deficient sparse a's, but never for classical
-	stopifnot(all.equal(drop(qr.qy (a.qr, qr.qty(a.qr, y))), y),
-		  all.equal(drop(qr.qty(a.qr, qr.qy (a.qr, y))), y))
+    if(Qinv.chk) { ## qr.qy and qr.qty should be inverses,  Q'Q y = y = QQ' y :
+	## FIXME: Fails for structurally rank deficient sparse a's, but never for classical
+	assert.EQ(drop(qr.qy (a.qr, qr.qty(a.qr, y))), y, giveRE=giveRE, tol = tol/64)
+	assert.EQ(drop(qr.qty(a.qr, qr.qy (a.qr, y))), y, giveRE=giveRE, tol = tol/64)
+    }
 
     piv <- switch(kind,
                   "qr" = a.qr$pivot,
@@ -53,10 +60,10 @@ chkQR <- function(a,
         rr <- if(cmpl) n else p
         stopifnot(dim(Q) == c(n,rr),
                   dim(R) == c(rr,p))
-        assert.EQ.Mat(a, Q %*% R[, invP], tol = tol)
+        assert.EQ.Mat(a, Q %*% R[, invP], giveRE=giveRE, tol=tol)
         ##            =  ===============
 	if(QtQ.chk)
-	    assert.EQ.mat(crossprod(Q), diag(rr), tol = tol)
+	    assert.EQ.mat(crossprod(Q), diag(rr), giveRE=giveRE, tol=tol)
         ##                ===========   ====
     }
     .ckQR(FALSE)
@@ -65,7 +72,7 @@ chkQR <- function(a,
 }## end{chkQR}
 
 ##' Check QR-consistency of dense and sparse
-chk.qr.D.S <- function(d., s., y, Y = Matrix(y), force = FALSE) {
+chk.qr.D.S <- function(d., s., y, Y = Matrix(y), force = FALSE, tol = 1e-10) {
     stopifnot(is.qr(d.), is(s., "sparseQR"))
     cc <- qr.coef(d.,y)
     rank.def <- any(is.na(cc)) && d.$rank < length(d.$pivot)
@@ -74,9 +81,12 @@ chk.qr.D.S <- function(d., s., y, Y = Matrix(y), force = FALSE) {
     ## when system is rank deficient, have differing cases, not always just NA <-> 0 coef
     ## FIXME though:  resid & fitted should be well determined
     if(force || !rank.def) stopifnot(
-	is.all.equal3(	    cc	     , drop(qr.coef  (s.,y)), drop(qr.coef  (s.,Y))) ,
-	is.all.equal3(qr.resid (d.,y), drop(qr.resid (s.,y)), drop(qr.resid (s.,Y))) ,
-	is.all.equal3(qr.fitted(d.,y), drop(qr.fitted(s.,y)), drop(qr.fitted(s.,Y)))
+### FIXME: temporary:
+###	is.all.equal3(	    cc	     , drop(qr.coef  (s.,y)), drop(qr.coef  (s.,Y)), tol=tol),
+	is.all.equal3(	unname( cc ) , drop(qr.coef  (s.,y)), drop(qr.coef  (s.,Y)), tol=tol),
+### END{FIXME}
+	is.all.equal3(qr.resid (d.,y), drop(qr.resid (s.,y)), drop(qr.resid (s.,Y)), tol=tol),
+	is.all.equal3(qr.fitted(d.,y), drop(qr.fitted(s.,y)), drop(qr.fitted(s.,Y)), tol=tol)
 	)
 }
 
@@ -91,23 +101,23 @@ chk.qr.D.S <- function(d., s., y, Y = Matrix(y), force = FALSE) {
 ##' @return list with 'qA' (sparse QR) and 'qa' (traditional (dense) QR)
 ##' @author Martin Maechler
 checkQR.DS.both <- function(A, Qinv.chk, QtQ.chk=NA,
-                            quiet=FALSE, tol = 1e-13)
+                            quiet=FALSE, giveRE=TRUE, tol = 1e-13)
 {
     stopifnot(is(A,"Matrix"))
     if(!quiet) cat("classical: ")
-    qa <- chkQR(as.matrix(A), Qinv.chk=TRUE, QtQ.chk=TRUE, tol=tol)# works always
+    qa <- chkQR(as.matrix(A), Qinv.chk=TRUE, QtQ.chk=TRUE, tol=tol, giveRE=giveRE)# works always
     if(!quiet) cat("[Ok] ---  sparse: ")
-    qA <- chkQR(A, Qinv.chk=Qinv.chk, QtQ.chk=QtQ.chk, tol=tol)
+    qA <- chkQR(A, Qinv.chk=Qinv.chk, QtQ.chk=QtQ.chk, tol=tol, giveRE=giveRE)
     validObject(qA)
     if(!quiet) cat("[Ok]\n")
-    chk.qr.D.S(qa, qA, y = 10 + 1:nrow(A))# ok [not done in rank deficient case!]
+    chk.qr.D.S(qa, qA, y = 10 + 1:nrow(A), tol = 256*tol)# ok [not done in rank deficient case!]
     invisible(list(qA=qA, qa=qa))
 }
 
 if(doExtras) system.time({ ## ~ 20 sec {"large" example}   + 2x qr.R() warnings
     cat("chkQR( <KNex> ) .. takes time .. ")
-    chkQR(mm, y=y, a.qr = Sq)
-    chkQR(md, y=y, a.qr = Dq)
+    chkQR(mm, y=y, a.qr = Sq, verbose=TRUE)
+    chkQR(md, y=y, a.qr = Dq, verbose=TRUE)
     cat(" done: [Ok]\n")
 })
 
@@ -116,18 +126,25 @@ chk.qr.D.S(Dq, Sq, y, Y)
 
 ## rank deficient QR cases: ---------------
 
-## From Simon (15 Jul 2009):
+## From Simon (15 Jul 2009) + dimnames (11 May 2015)
 set.seed(10)
-a <- matrix(round(10 * runif(90)), 10,9); a[a < 7.5] <- 0
+a <- matrix(round(10 * runif(90)), 10,9, dimnames =
+            list(LETTERS[1:10], paste0("c", 1:9)))
+a[a < 7.5] <- 0
 (A <- Matrix(a))# first column = all zeros
-qD <- chkQR(a) ## using base qr
-qS <- chkQR(A) ## using Matrix "sparse qr" -- "structurally rank deficient!
+qD <- chkQR(a, giveRE=TRUE) ## using base qr
+qS <- chkQR(A, giveRE=TRUE) ## using Matrix "sparse qr" -- "structurally rank deficient!
 validObject(qS)# with the validity now (2012-11-18) -- ok, also for "bad" case
+## Here, have illegal access  Up[-1] in ../src/cs.c
+try( ## After patch (2016-10-04 - *NOT* committed), this fails
+    ## definitely "fails" (with good singularity message) after c3194 (cs.c):
 chk.qr.D.S(qD, qS, y = 10 + 1:nrow(A), force=TRUE)# 6 warnings: "structurally rank deficient"
+)
 try( ## NOTE: *Both* checks  currently fail here:
     chkQR(A, Qinv.chk=TRUE, QtQ.chk=TRUE)
 )
 
+
 ## Larger Scale random testing
 oo <- options(Matrix.quiet.qr.R = TRUE, Matrix.verbose = TRUE)
 set.seed(101)
@@ -135,8 +152,9 @@ set.seed(101)
 for(N in 1:(if(doExtras) 1008 else 24)) {
     A <- rsparsematrix(8,5, nnz = rpois(1, lambda=16))
     cat(sprintf("%4d -", N))
-    checkQR.DS.both(A, Qinv.chk= NA, QtQ.chk=NA)
+    checkQR.DS.both(A, Qinv.chk= NA, QtQ.chk=NA,
     ##                          --- => FALSE if struct. rank deficient
+		    giveRE = FALSE, tol = if(is64bit) 2e-13 else 1e-12)
 }
 unique(warnings())
 
@@ -212,11 +230,11 @@ for(N in 1:(if(doExtras) 2^12 else 128)) {
     if(any(apply(A, 2, function(x) all(x == 0)))) ## "column of all 0"
         next
     cat(sprintf("%4d -", N))
-    checkQR.DS.both(A, Qinv.chk=NA, tol = 1e-12)
+    checkQR.DS.both(A, Qinv.chk=NA, giveRE=FALSE, tol = 1e-12)
     ##                         --- => FALSE if struct. rank deficient
 }
 
-1
+unique(warnings())
 
 
 options(oo)
