@@ -329,7 +329,7 @@ if(doExtras) {### {was ./AAA_index.R, MM-only}
     stopifnot(all(C. == N))
 
     print(load(system.file("external", "symA.rda", package="Matrix"))) # "As"
-    stopifnot(isValid(As, "dsCMatrix"), identical(As@factors, list()))
+    stopifnotValid(As, "dsCMatrix"); stopifnot(identical(As@factors, list()))
     R. <- drop0(chol(As))
     stopifnot(1:32 == sort(diag(R.)), ## !
               R.@x == as.integer(R.@x),## so it is an integer-valued chol-decomp !
@@ -358,7 +358,7 @@ if(doExtras) {### {was ./AAA_index.R, MM-only}
 ##---- Symmetric indexing of symmetric Matrix ----------
 m. <- mC
 m.[, c(2, 7:12)] <- 0
-isValid(S <- crossprod(add.simpleDimnames(m.) %% 100), "dsCMatrix")
+stopifnotValid(S <- crossprod(add.simpleDimnames(m.) %% 100), "dsCMatrix")
 ss <- as(S, "matrix")
 ds <- as(S, "denseMatrix")
 ## NA-indexing of *dense* Matrices: should work as traditionally
@@ -540,7 +540,7 @@ for(i in 1:(if(doExtras) 200 else 25)) {
     assertWarning(Nn[1:2,] <- -pi)
     assertWarning(Nn[, 5] <- -pi)
     assertWarning(Nn[2:4, 5:8] <- -pi)
-    stopifnot(isValid(Nn,"nsparseMatrix"))
+    stopifnotValid(Nn,"nsparseMatrix")
     ##
     cat(".")
     if(i %% 10 == 0) cat("\n")
@@ -567,12 +567,15 @@ stopifnot(identical(m0[2,], m0[,2]),
 (m1 <- as(m0, "TsparseMatrix")) # dtTMatrix unitriangular
 (m2 <- as(m0, "CsparseMatrix")) # dtCMatrix unitriangular
 m1g <- as(m1, "generalMatrix")
-stopifnot(is(m1g, "dgTMatrix"))
+tr1 <- as(m1, "denseMatrix") # dtrMatrix unitriangular
+stopifnotValid(m1g, "dgTMatrix")
+diag(tr1) <- 100
+stopifnot(diag(tr1) == 100)# failed when 'diag<-' did not recycle
 assert.EQ.mat(m2[1:3,],    diag(5)[1:3,])
 assert.EQ.mat(m2[,c(4,1)], diag(5)[,c(4,1)])
 stopifnot(identical(m2[1:3,], as(m1[1:3,], "CsparseMatrix")),
-          identical(Matrix:::uniqTsparse(m1[, c(4,2)]),
-                    Matrix:::uniqTsparse(as(m2[, c(4,2)], "TsparseMatrix")))
+          identical(uniqTsparse(m1[, c(4,2)]),
+                    uniqTsparse(as(m2[, c(4,2)], "TsparseMatrix")))
           )## failed in 0.9975-11
 
 (uTr <- new("dtTMatrix", Dim = c(3L,3L), diag="U"))
@@ -638,8 +641,10 @@ D[i] <- v; assert.EQ.mat(D,m) # ddi -> dtT -> dgT
 s[i] <- v; assert.EQ.mat(s,m) # dtT -> dgT
 S[i] <- v; assert.EQ.mat(S,m); S # dtC -> dtT -> dgT -> dgC
 m.L <- asLogical(m)
-C[i] <- v; assert.EQ.mat(C,m.L); validObject(C)
-N[i] <- v; assert.EQ.mat(N,m.L); validObject(N)
+C[i] <- v # - with a warning: C is nMatrix, v not T/F
+assert.EQ.mat(C,m.L); validObject(C)
+N[i] <- v # - with a warning
+assert.EQ.mat(N,m.L); validObject(N)
 stopifnot(Q.C.identical(D,s, checkClass=FALSE))
 ## logical *vector* indexing
 eval(.iniDiag.example)
@@ -648,7 +653,7 @@ M[L] <- z; assert.EQ.mat(M,m)
 D[L] <- z; assert.EQ.mat(D,m)
 s[L] <- z; assert.EQ.mat(s,m)
 S[L] <- z; assert.EQ.mat(S,m) ; S
-C[L] <- z; assert.EQ.mat(C,m.L)
+C[L] <- z; assert.EQ.mat(C,m.L) # with a good warning
 N[L] <- z; assert.EQ.mat(N,m.L)
 
 
@@ -702,10 +707,13 @@ M[i] <- v; assert.EQ.mat(M,m) # dge
 D[i] <- v; assert.EQ.mat(D,m) # ddi -> dtT -> dgT
 s[i] <- v; assert.EQ.mat(s,m) # dtT -> dgT
 S[i] <- v; assert.EQ.mat(S,m); S # dtC -> dtT -> dgT -> dgC
-N[i] <- v; assert.EQ.mat(N,m.L); N #
-C[i] <- v; assert.EQ.mat(C,m.L); C #
-
+N[i] <- v # with a good warning
+assert.EQ.mat(N,m.L); N
+C[i] <- v #  ..  .  ..  warning
+assert.EQ.mat(C,m.L); C #
 
+options(warn = 2) #---------------------# NO WARNINGS from here -----------------
+					# =====================
 ## 2) negative [i,j] indices
 mc <- mC[1:5, 1:7]
 mt <- mT[1:5, 1:7]
@@ -1036,4 +1044,75 @@ for(nm in ls()) if(is(.m <- get(nm), "Matrix")) {
 showProc.time()
 }#--------------end if(doExtras) -----------------------------------------------
 
+## Bugs found by Peter Ralph
+n <- 17
+x <- Matrix(0, n,n)
+## x must have at least three nonzero entries
+x[1,1] <- x[2,1:2] <- 1.
+x0 <- x <- as(x,"dgTMatrix")  # if x is dgCMatrix, no error
+##
+z <- matrix(x) # <== not the "Matrix way":  a (n, 1) matrix
+z[1] <- 0
+
+x[1:n, 1:n] <- as(z, "sparseVector")
+## gave Error: ... invalid subscript type 'S4'
+x2 <- x
+
+dim(zC <- as(z, "dgCMatrix"))
+x <- x0
+x[] <- zC # did fail, then gave warning.
+x1 <- x
+##
+x <- x0
+x[] <- as(zC, "sparseVector") # did fail, too
+x2 <- x
+stopifnot(identical(x1,x2))
+x <- as(x0, "matrix")
+x[] <- z
+assert.EQ.mat(x1, x)
+
+i <- 4:7
+x1 <- x0; x1[cbind(i, i+10)] <- i^2
+x2 <- x0; x2[cbind(i, i+10)] <- as.matrix(i^2)
+## failed: nargs() = 4 ... please report
+
+stopifnot(isValid(x1, "dgTMatrix"), identical(x1, x2))
+
+
+iv <- c(rep(0,100), 3, 0,0,7,0,0,0)
+sv0  <- sv  <- as(iv, "sparseVector")
+sv.0 <- sv. <- as(as.integer(iv), "sparseVector")
+stopifnot(canCoerce("integer", "sparseVector"))
+sv2 <- as(sv, "isparseVector")
+stopifnot(validObject(sv), validObject(sv2), identical(sv., sv2),
+          sv == sv.)
+n0 <- sv. != 0
+## --> ../R/sparseVector.R : replSPvec()
+if(interactive())  debug(Matrix:::replSPvec)
+##
+sv [n0] <- sv [n0]
+sv.[n0] <- sv.[n0] # gave error
+stopifnot(identical(sv , sv0),
+          identical(sv., sv.0))
+sv [3:7] <- 0
+sv.[3:7] <- 0L
+stopifnot(identical(sv , sv0), identical(sv., sv.0))
+sv [2:4] <- 2:4
+sv.[2:4] <- 2:4
+stopifnot(which(sv != 0) == (which(sv. != 0) -> in0),
+          in0 == c(2:4, 101L, 104L))
+sv [2:6] <- 0L
+sv.[2:6] <- 0L
+stopifnot(identical(sv , sv0), identical(sv., sv.0))
+
+## the next six *all* gave an error -- but should be no-op's:
+for(vv in list(sv, sv.0))
+    for(ind in list(0, FALSE, logical(length(vv))))
+        vv[ind] <- NA
+stopifnot(identical(sv , sv0), identical(sv., sv.0))
+
+
+
+showProc.time()
+
 if(!interactive()) warnings()
